
/*******************************************************************************
*
* File check2.c
*
* Copyright (C) 2014 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Inventory of source fields.
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <limits.h>
#include "utils.h"
#include "permutations.h"
#include "rvt.h"
#include "contract.h"

static int k,n,ml,mj,cidl,cidj;
static int *ki,*li,*ji,*ne0;
static int *nl,*nj,(*rl)[2],(*rj)[2];
static long nck,ncl,ncj;
static long nmk,nml,nmj;
static vlist_t vls;
static ltree_t ***lta,**ltd;
static tlist_t ***tls;


static theory_t theory(char *name)
{
   if (strcmp("PHI4",name)==0)
      return PHI4;
   else if (strcmp("YM",name)==0)
      return YM;
   else if (strcmp("QCD",name)==0)
      return QCD;
   else if (strcmp("LYM",name)==0)
      return LYM;
   else if (strcmp("LQCD",name)==0)
      return LQCD;
   else
      error(1,1,"theory [check2.c]","Unknown theory");

   return THEORIES;
}


static void set_tls(theory_t th,int order)
{
   int i,j,no,nl,nlp;

   no=order+1;
   nl=0;

   for (i=1;i<=order;i++)
      nl+=(nlp_max(th,i)+1);

   tls=malloc(no*sizeof(*tls));
   error(tls==NULL,1,"set_tls [check2.c]","Unable to allocate tlist array");

   tls[0]=NULL;
   tls[1]=malloc(nl*sizeof(**tls));
   error(tls[1]==NULL,2,"set_tls [check2.c]","Unable to allocate tlist array");

   for (i=2;i<=order;i++)
      tls[i]=tls[i-1]+nlp_max(th,i-1)+1;

   for (i=1;i<=order;i++)
   {
      nlp=nlp_max(th,i);

      for (j=0;j<=nlp;j++)
         tls[i][j]=select_ltrees(th,i,j);
   }
}


static void set_nl(void)
{
   int i;

   ml=0;
   nl[0]=1;

   for (i=1;i<n;i++)
   {
      if (ki[i]==ki[i-1])
         nl[ml]+=1;
      else
      {
         ml+=1;
         nl[ml]=1;
      }
   }

   ml+=1;
}


static void set_rl(theory_t th)
{
   int i,j;

   j=0;

   for (i=0;i<ml;i++)
   {
      rl[i][0]=0;
      rl[i][1]=nlp_max(th,ki[j]);
      j+=nl[i];
   }
}


static void set_nj(void)
{
   int i;

   mj=0;
   nj[0]=1;

   for (i=1;i<n;i++)
   {
      if ((ki[i]==ki[i-1])&&(li[i]==li[i-1]))
         nj[mj]+=1;
      else
      {
         mj+=1;
         nj[mj]=1;
      }
   }

   mj+=1;
}


static int set_rj(theory_t th)
{
   int i,j;
   tlist_t *tl;

   j=0;

   for (i=0;i<mj;i++)
   {
      rj[i][0]=0;
      tl=tls[ki[j]][li[j]];

      if (tl!=NULL)
         rj[i][1]=(*tl).n-1;
      else
         return 0;

      j+=nj[i];
   }

   return 1;
}


static void set_lta(void)
{
   int i;
   tlist_t *tl;

   for (i=0;i<n;i++)
   {
      tl=tls[ki[i]][li[i]];
      lta[i]=(*tl).lta;
   }
}


static void set_ltd(void)
{
   int i;

   for (i=0;i<n;i++)
      ltd[i]=lta[i][ji[i]];
}


static void fnd_idx(int il,line_t *ln,src_legs_t *sl,int *idx)
{
   int i,lbl,nt,*slbl;
   ltype_t lt,lr,*slt;

   lt=ln[il].type;
   lr=rvt[(int)(lt)];
   lbl=1;

   for (i=0;i<il;i++)
   {
      if (ln[i].type==lt)
         lbl+=1;
   }

   nt=(*sl).nt;
   slbl=(*sl).lbl;
   slt=(*sl).lt;
   idx[0]=nt;
   idx[1]=nt;

   for (i=1;i<nt;i++)
   {
      if ((lt==slt[i])&&(lbl==slbl[i]))
         idx[0]=i;

      if ((lr==slt[i])&&(lbl==slbl[i]))
         idx[1]=i;
   }

   error((idx[0]==nt)||(idx[1]==nt),1,"fnd_idx [check2.c]",
         "Unable to find source type");
}


static void add_lbl2zero(src_legs_t *sl)
{
   int nt,i,j,l;
   ltype_t lt;

   nt=(*sl).nt;
   lt=(*sl).lt[0];

   for (i=0;i<(*sl).nv[0];i++)
   {
      j=(*sl).iv[0][i];
      ne0[j]=(*sl).ne[0][i];
   }

   for (i=1;i<nt;i++)
   {
      if (lt==(*sl).lt[i])
      {
         for (j=0;j<(*sl).nv[i];j++)
         {
            l=(*sl).iv[i][j];
            ne0[l]+=(*sl).ne[i][j];
         }
      }
   }
}


static void add_int2zero(src_legs_t *sl)
{
   int nv,nln,rv;
   int i,il,iv0,iv1;
   ltype_t lt;
   line_t *ln;
   diagram_t *dm;

   lt=(*sl).lt[0];
   nv=0;

   for (i=0;i<n;i++)
   {
      dm=ltd[i][0].dm;
      nln=(*dm).nl;
      ln=(*dm).l;

      for (il=0;il<nln;il++)
      {
         if (lt==ln[il].type)
         {
            iv0=ln[il].iv[0]+nv;
            iv1=ln[il].iv[1]+nv;

            ne0[iv0]+=1;
            ne0[iv1]+=1;
         }
      }

      rv=ltd[i][0].rv;

      if (rv<(*dm).nv)
         ne0[rv+nv]+=1;

      nv+=(*dm).nv;
   }
}


static void cmp_zero(src_legs_t *sl)
{
   int nv,nvd,ivd;
   int i,(*v)[2];
   ltype_t lt;
   vertex_t *vx;
   diagram_t *dm;

   lt=(*sl).lt[0];
   nv=0;

   for (i=0;i<n;i++)
   {
      dm=ltd[i][0].dm;
      nvd=(*dm).nv;
      v=(*dm).v;

      for (ivd=0;ivd<nvd;ivd++)
      {
         vx=vls.v[v[ivd][0]]+v[ivd][1];

         error((*vx).nl[(int)(lt)]!=ne0[ivd+nv],1,
               "cmp_zero [check2.c]",
               "Incorrect number of label 0 source lines");
      }

      nv+=nvd;
   }
}


static void rm_lns(src_legs_t *sl)
{
   int nv,nln;
   int il,iv0,iv1;
   int i,j,idx[2];
   line_t *ln;
   diagram_t *dm;

   nv=0;

   for (i=0;i<n;i++)
   {
      nln=ltd[i][0].nl;
      ln=ltd[i][0].l;

      for (il=0;il<nln;il++)
      {
         fnd_idx(il,ln,sl,idx);
         iv0=ln[il].iv[0]+nv;
         iv1=ln[il].iv[1]+nv;

         for (j=0;j<(*sl).nv[idx[0]];j++)
         {
            if (iv0==(*sl).iv[idx[0]][j])
            {
               (*sl).ne[idx[0]][j]-=1;
               break;
            }
         }

         error(j==(*sl).nv[idx[0]],1,"rm_lns [check2.c]",
               "Vertex not found");

         for (j=0;j<(*sl).nv[idx[1]];j++)
         {
            if (iv1==(*sl).iv[idx[1]][j])
            {
               (*sl).ne[idx[1]][j]-=1;
               break;
            }
         }

         error(j==(*sl).nv[idx[1]],2,"rm_lns [check2.c]",
               "Vertex not found");
      }

      dm=ltd[i][0].dm;
      nv+=(*dm).nv;
   }
}


static void chk_src_legs(theory_t th,src_legs_t *sl)
{
   int nvt,nt,i,j;
   int **iv,**ne,*nv;
   diagram_t *dm;

   nvt=0;

   for (i=0;i<n;i++)
   {
      dm=ltd[i][0].dm;
      nvt+=(*dm).nv;
   }

   for (i=0;i<nvt;i++)
      ne0[i]=0;

   nt=(*sl).nt;
   nv=(*sl).nv;
   iv=(*sl).iv;
   ne=(*sl).ne;

   for (i=0;i<nt;i++)
   {
      for (j=0;j<nv[i];j++)
      {
         if ((iv[i][j]<0)||(iv[i][j]>=nvt))
            error(1,1,"chk_src_legs [check2.c]",
                  "Vertex indices are out of range");

         if (ne[i][j]<1)
            error(1,1,"chk_src_legs [check2.c]",
                  "Numbers of external lines are out of range");
      }
   }

   error(((th==PHI4)&&((*sl).lt[0]!=SCALAR))||
         ((th!=PHI4)&&((*sl).lt[0]!=GLUON)),1,"chk_src_legs [check2.c]",
                  "Incorrect label=0 field type");

   add_lbl2zero(sl);
   add_int2zero(sl);
   cmp_zero(sl);
   rm_lns(sl);

   for (i=1;i<nt;i++)
   {
      for (j=0;j<nv[i];j++)
      {
         if (ne[i][j]!=0)
            error(1,1,"chk_src_legs [check2.c]",
                  "Incorrect numbers of labeled sources");
      }
   }
}


int main(void)
{
   int order,is,na0,na1;
   char name[5];
   theory_t th;
   src_legs_t sl;

   printf("\n");
   printf("Inventory of source fields\n");
   printf("--------------------------\n\n");

   init_src_legs(&sl);

   while (1)
   {
      printf("Specify theory, maximal order and root selection flag: ");
      (void)(scanf("%4s %d %d",name,&order,&is));

      th=theory(name);
      na0=trees(th,order);
      na1=ltrees(th,order);
      set_root(th,is);
      set_tls(th,order);
      vls=vertices(th,order);

      printf("\n");
      printf("Total number of basic tree diagrams =   %d\n",na0);
      printf("Total number of labeled tree diagrams = %d\n",na1);

      n=order+1;
      ki=malloc(6*n*sizeof(*ki));
      rl=malloc(2*n*sizeof(*rl));
      lta=malloc(n*sizeof(*lta));
      ltd=malloc(n*sizeof(*ltd));
      error((ki==NULL)||(rl==NULL)||(lta==NULL)||(ltd==NULL),1,
            "main [check2.c]","Unable to allocate auxiliary arrays");
      li=ki+n;
      ji=li+n;
      nl=ji+n;
      nj=nl+n;
      ne0=nj+n;
      rj=rl+n;

      for (k=1;k<=order;k++)
      {
         for (n=1;n<=k;n++)
         {
            nmk=init_partition(n,k);
            nck=0L;

            while (nck<nmk)
            {
               nck=next_partition(ki);
               set_nl();
               set_rl(th);
               nml=init_mindex(ml,nl,rl,&cidl);
               ncl=0L;

               while (ncl<nml)
               {
                  ncl=next_mindex(cidl,li);
                  set_nj();

                  if (set_rj(th))
                  {
                     set_lta();
                     nmj=init_mindex(mj,nj,rj,&cidj);
                     ncj=0L;

                     while (ncj<nmj)
                     {
                        ncj=next_mindex(cidj,ji);
                        set_ltd();
                        get_src_legs(th,n,ltd,&sl);
                        chk_src_legs(th,&sl);
                     }

                     stop_mindex(cidj);
                  }
               }

               stop_mindex(cidl);
            }

            stop_partition();
         }
      }

      printf("No errors found\n\n");
   }

   exit(0);
}
