
/*******************************************************************************
*
* File check4.c
*
* Copyright (C) 2014 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Trivializing map.
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "utils.h"
#include "contract.h"


static theory_t theory(char *name)
{
   if (strcmp("PHI4",name)==0)
      return PHI4;
   else if (strcmp("YM",name)==0)
      return YM;
   else if (strcmp("QCD",name)==0)
      return QCD;
   else if (strcmp("LYM",name)==0)
      return LYM;
   else if (strcmp("LQCD",name)==0)
      return LQCD;
   else
      error(1,1,"theory [check4.c]","Unknown theory");

   return THEORIES;
}


static void print_dgmno1(theory_t th,int order)
{
   int i,j,k,l,nlp;
   int sm0[6],sm1[6];
   cnum_t **cna;
   vlist_t vl;
   ltree_t *ltd,**lta;
   tlist_t *tl;

   printf("The numbers of diagrams at fixed order in the coupling are:\n\n");

   printf(" order    all   c(R)!=0    ");

   if (th!=PHI4)
      printf(" w/o gfix  c(R)!=0    w/o gfix&ghost  c(R)!=0\n");
   else
      printf("\n");

   printf("-------------------------");

   if (th!=PHI4)
      printf("-------------------------------------------------\n");
   else
      printf("\n");

   for (l=0;l<6;l++)
      sm0[l]=0;

   vl=vertices(th,order);

   for (i=1;i<=order;i++)
   {
      for (l=0;l<6;l++)
         sm1[l]=0;

      nlp=nlp_max(th,i);

      for (j=0;j<=nlp;j++)
      {
         tl=select_ltrees(th,i,j);

         if (tl!=NULL)
         {
            lta=(*tl).lta;
            cna=(*tl).cna;

            for (k=0;k<(*tl).n;k++)
            {
               ltd=lta[k];

               sm1[0]+=1;
               sm1[1]+=is_zero_cnum(cna[k]);

               if (has_gfix((*ltd).dm,&vl)==0)
               {
                  sm1[2]+=1;
                  sm1[3]+=is_zero_cnum(cna[k]);

                  if (has_ghost((*ltd).dm,&vl)==0)
                  {
                     sm1[4]+=1;
                     sm1[5]+=is_zero_cnum(cna[k]);
                  }
               }
            }
         }
      }

      printf("  %2d  %7d  %7d",i,sm1[0],sm1[0]-sm1[1]);

      if (th!=PHI4)
         printf("     %7d   %7d       %7d      %7d\n",
                sm1[2],sm1[2]-sm1[3],sm1[4],sm1[4]-sm1[5]);
      else
         printf("\n");

      for (l=0;l<6;l++)
         sm0[l]+=sm1[l];
   }

   printf("-------------------------");

   if (th!=PHI4)
      printf("-------------------------------------------------\n");
   else
      printf("\n");

   printf("      %7d  %7d",sm0[0],sm0[0]-sm0[1]);

   if (th!=PHI4)
      printf("     %7d   %7d       %7d      %7d\n\n",
             sm0[2],sm0[2]-sm0[3],sm0[4],sm0[4]-sm0[5]);
   else
      printf("\n\n");
}


static void print_dgmno2(theory_t th,int order)
{
   int i,j,k,l,nlp;
   int sm0[3],sm1[3];
   cnum_t **cna;
   vlist_t vl;
   ltree_t *ltd,**lta;
   tlist_t *tl;

   printf("Number of diagrams with c(R)!=0 and 2 or more branches:\n\n");

   printf(" order    all    ");

   if (th!=PHI4)
      printf(" w/o gfix    w/o gfix&ghost\n");
   else
      printf("\n");

   printf("-----------------");

   if (th!=PHI4)
      printf("-----------------------------\n");
   else
      printf("\n");

   for (l=0;l<3;l++)
      sm0[l]=0;

   vl=vertices(th,order);

   for (i=1;i<=order;i++)
   {
      for (l=0;l<3;l++)
         sm1[l]=0;

      nlp=nlp_max(th,i);

      for (j=0;j<=nlp;j++)
      {
         tl=select_ltrees(th,i,j);

         if (tl!=NULL)
         {
            lta=(*tl).lta;
            cna=(*tl).cna;

            for (k=0;k<(*tl).n;k++)
            {
               ltd=lta[k];

               if ((is_zero_cnum(cna[k])!=1)&&
                   (is_branch((*ltd).dm)==0))
               {
                  sm1[0]+=1;

                  if (has_gfix((*ltd).dm,&vl)==0)
                  {
                     sm1[1]+=1;

                     if (has_ghost((*ltd).dm,&vl)==0)
                        sm1[2]+=1;
                  }
               }
            }
         }
      }

      printf("  %2d  %7d",i,sm1[0]);

      if (th!=PHI4)
         printf("    %7d        %7d\n",sm1[1],sm1[2]);
      else
         printf("\n");

      for (l=0;l<3;l++)
         sm0[l]+=sm1[l];
   }

   printf("-----------------");

   if (th!=PHI4)
      printf("-----------------------------\n");
   else
      printf("\n");

   printf("      %7d",sm0[0]);

   if (th!=PHI4)
      printf("    %7d        %7d\n\n",sm0[1],sm0[2]);
   else
      printf("\n\n");
}


static void print_heights(theory_t th,int order)
{
   int i,j,nel,h;
   dlist_t *dl;

   printf("Height of the search trees:\n");

   for (i=1;i<=order;i++)
   {
      nel=nel_max(th,i);

      for (j=2;j<=nel;j++)
      {
         dl=select_cdgms(th,i,j);

         if (dl!=NULL)
         {
            h=check_aatree((*dl).tid);
            printf("Height at order %d and %d external lines = %d\n",i,j,h);
         }
      }
   }

   printf("\n");
}


int main(void)
{
   int order,is,na0;
   int n,i,j,k,nlp;
   double dt;
   clock_t t1,t2;
   cnum_t **cna;
   char name[5];
   theory_t th;
   tlist_t *tl;


   printf("\n");
   printf("Trivializing map\n");
   printf("----------------\n\n");

   while (1)
   {
      printf("Specify theory, maximal order and root selection flag: ");
      (void)(scanf("%4s %d %d",name,&order,&is));
      printf("\n");

      th=theory(name);
      t1=clock();
      na0=tmap(th,order,is);
      t2=clock();
      dt=(double)(t2-t1)/(double)(CLOCKS_PER_SEC);

      printf("Trivializing map obtained in %.1e seconds\n",dt);

      printf("Total number of labeled tree diagrams = %d\n\n",na0);

      print_dgmno1(th,order);
      print_dgmno2(th,order);
      print_heights(th,order);

      printf("Print coefficients? (1|0) ");
      (void)(scanf("%d",&is));

      if (is==1)
      {
         for (i=1;i<=order;i++)
         {
            nlp=nlp_max(th,i);

            for (j=0;j<=nlp;j++)
            {
               tl=select_ltrees(th,i,j);

               if (tl!=NULL)
               {
                  n=(*tl).n;
                  cna=(*tl).cna;

                  printf("\n");
                  printf("order = %d, nloop = %d, no of diagrams = %d\n",
                         i,j,n);

                  for (k=0;k<n;k++)
                  {
                     printf("c(R_%d) = ",k);
                     print_cnum(stdout,"Nf",cna[k]);
                     printf("\n");
                  }
               }
            }
         }
      }

      printf("\n");
   }

   exit(0);
}
